#include <cassert>
#include "App.h"
#include "Engine.h"

void App::ProcessEvents() {
  if (is_done) {
    return;
  }

  // przyjrzyj zdarzenia
  SDL_Event event;
  while (SDL_PollEvent(&event)) {
    if (event.type == SDL_VIDEORESIZE) {
      Resize(event.resize.w, event.resize.h);
    } else if (event.type == SDL_QUIT) {
      is_done = true;
      break;
    // } else if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE) {
    //   is_done = true;
    //   break;
    }
  }
}

void App::Run() {
  // inicjalizacja okna
  SDL_Init(SDL_INIT_VIDEO);
  Resize(m_window_width, m_window_height);
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1); // podwójne buforowanie

  // inicjalizacja OpenGL
  glClearColor(0, 0, 0, 0);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_TEXTURE_2D);

  const std::string test_sprite_filename = "data/counter.bmp";
  Engine& engine = Engine::Get();
  engine.Load();
  engine.Renderer()->LoadTexture(test_sprite_filename);
  m_litery.reset(new Sprite(engine.SpriteConfig()->Get("litery")));
  m_liczby.reset(new Sprite(engine.SpriteConfig()->Get("liczby")));
  m_liczby_loop.reset(new Sprite(engine.SpriteConfig()->Get("liczby-loop")));

  // pętla główna
  is_done = false;
  size_t last_ticks = SDL_GetTicks();
  while (!is_done) {
    ProcessEvents();

    // time update
    size_t ticks = SDL_GetTicks();
    double delta_time = (ticks - last_ticks) / 1000.0;
    last_ticks = ticks;

    // update & render
    if (delta_time > 0) {
      Update(delta_time);
    }
    Draw();
  }

  SDL_Quit();
}

void App::Update(double dt) {
  m_litery->Update(dt);
  m_liczby->Update(dt);
  m_liczby_loop->Update(dt);
}

void App::Draw() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();

  m_litery->DrawCurrentFrame(0, 0, .5, .5); // x, y, width, height
  m_liczby->DrawCurrentFrame(.5, 0, .3, .3);
  m_liczby_loop->DrawCurrentFrame(.5, .5, .4, .4);

  SDL_GL_SwapBuffers();
}

void App::Resize(size_t width, size_t height) {
  m_screen = SDL_SetVideoMode(width, height, 32, SDL_OPENGL | SDL_RESIZABLE | SDL_HWSURFACE);
  assert(m_screen && "problem z ustawieniem wideo");
  m_window_width = width;
  m_window_height = height;

  glViewport(0, 0, static_cast<GLsizei> (width), static_cast<GLsizei> (height));
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 1, 0, 1, -1, 10);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}
