#ifndef SPRITECONFIG_H_
#define SPRITECONFIG_H_

#include <map>
#include <string>
#include <boost/shared_ptr.hpp>

struct SpriteConfigData {
  /**
   * frame_duration - czas trwania klatki (w sekundach)
   * left, bottom, width oraz height są współrzędnymi klatki w pikselach (na obrazku) oraz jej rozmiarami.
   * level - poziom, na którym będzie wyświetlony sprite
   */
  explicit SpriteConfigData(size_t level, size_t frame_count, double frame_duration, double left,
      double bottom, double width, double height, bool loop) :
    level(level), frame_count(frame_count), frame_duration_time(frame_duration), left(left), bottom(bottom),
        width(width), height(height), loop(loop) {

  }

    size_t level;        // plan, na którym będzie rysowany sprite. Im bliżej 0, tym bliżej obserwatora
    size_t frame_count;  // liczba klatek w animacji
    double frame_duration_time;   // czas trwania klatki
    double left;   // położenie w poziomie pierwszej klatki animacji w obrazku (w px)
    double bottom; // położenie w pionie pierwszej klatki animacji w obrazku (w px)
    double width;  // szerokość klatki w pikselach
    double height; // wysokość klatki w pikselach
    bool loop;     // czy animacja ma być zapętlona?
};

class SpriteConfig {
public:
  explicit SpriteConfig();
  SpriteConfigData Get(const std::string& name) const;
  bool Contains(const std::string& name) const;

private:
  std::map<std::string, SpriteConfigData> m_data;
  void Insert(const std::string& name, const SpriteConfigData& data);
};

typedef boost::shared_ptr<SpriteConfig> SpriteConfigPtr;

#endif /* SPRITECONFIG_H_ */
